package org.mrpdaemon.sec.encfs;

import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.KeySpec;
import java.util.Arrays;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.PBEKeySpec;

public class EncFSPBKDF2Data
{
	private static final SecretKeyFactory skf;

	static {
		try {
			skf = SecretKeyFactory.getInstance("PBKDF2WithHmacSHA1");
		} catch (NoSuchAlgorithmException ex) {
			throw new RuntimeException(ex);
		}
	}

	private String password;
	private byte[] pbkdf2Data;
	private byte[] passKeyData;
	private byte[] passIvData;

	private EncFSPBKDF2Data() { }

	public static EncFSPBKDF2Data from(EncFSBinaryConfig config, String password)
			throws InvalidKeySpecException {
		byte[] cipherSaltData = config.getSaltData();
		int keySizeInBits = config.getConfig().getVolumeKeySizeInBits();
		KeySpec ks = new PBEKeySpec(password.toCharArray(), cipherSaltData,
				config.getConfig().getIterationForPasswordKeyDerivationCount(),
				keySizeInBits + EncFSVolume.IV_LENGTH_IN_BYTES * 8);
		byte[] pbkdf2Data = skf.generateSecret(ks).getEncoded();

		EncFSPBKDF2Data obj;
		obj = EncFSPBKDF2Data.from(config.getConfig(), pbkdf2Data);
		obj.password = password;
		return obj;
	}

	public static EncFSPBKDF2Data from(EncFSConfig config, byte[] pbkdf2Data)
			throws InvalidKeySpecException {
		EncFSPBKDF2Data obj = new EncFSPBKDF2Data();
		int keySizeInBytes = config.getVolumeKeySizeInBits() / 8;
		obj.pbkdf2Data = pbkdf2Data;
		obj.passKeyData = Arrays.copyOfRange(pbkdf2Data, 0, keySizeInBytes);
		obj.passIvData = Arrays.copyOfRange(pbkdf2Data, keySizeInBytes,
				keySizeInBytes + EncFSVolume.IV_LENGTH_IN_BYTES);

		return obj;
	}

	public String getPassword() {
		return password;
	}

	public byte[] getPbkdf2Data() {
		return pbkdf2Data;
	}

	public byte[] getPassKeyData() {
		return passKeyData;
	}

	public byte[] getPassIvData() {
		return passIvData;
	}
}
